#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2022/7/19 15:08
# @Author  : zhangbc0315@outlook.com
# @File    : rxn_template_extractor.py
# @Software: PyCharm


class RxnTemplateExtractor:

    @classmethod
    def rxn_smiles_to_rt_smarts(cls, rxn_smiles: str):
        pass


if __name__ == "__main__":
    pass
